@@ -16,15 +16,48 @@
 package config
 
 import (
+	"bytes"
+	"context"
+	"crypto/rand"
+	"crypto/rsa"
+	"crypto/tls"
+	"crypto/x509"
+	"crypto/x509/pkix"
+	"encoding/json"
+	"encoding/pem"
 	"fmt"
+	"math/big"
+	"net"
+	"net/http"
+	"net/http/httptest"
 	"net/url"
 	"reflect"
 	"testing"
+	"time"
 
+	"github.com/coreos/go-oidc/v3/oidc"
+	lru "github.com/hashicorp/golang-lru/v2"
 	"github.com/sigstore/fulcio/pkg/generated/protobuf"
 )
 
-var validCfg = `
+var validYamlCfg = `
+oidc-issuers:
+  https://accounts.google.com:
+    issuer-url: https://accounts.google.com
+    client-id: foo
+    type: email
+    challenge-claim: email
+meta-issuers:
+  https://oidc.eks.*.amazonaws.com/id/*:
+    client-id: bar
+    type: kubernetes
+  https://oidc.foo.*.bar.com/id/*:
+    client-id: bar
+    type: ci-provider
+    ci-provider: github-workflow
+`
+
+var validJSONCfg = `
 {
 	"OIDCIssuers": {
 		"https://accounts.google.com": {
@@ -38,6 +71,11 @@ var validCfg = `
 		"https://oidc.eks.*.amazonaws.com/id/*": {
 			"ClientID": "bar",
 			"Type": "kubernetes"
+		},
+		"https://oidc.foo.*.bar.com/id/*": {
+			"ClientID": "bar",
+			"Type": "ci-provider",
+			"CiProvider": "github-workflow"
 		}
 	}
 }
@@ -489,6 +527,18 @@ func Test_issuerToChallengeClaim(t *testing.T) {
 	if claim := issuerToChallengeClaim(IssuerTypeGithubWorkflow, ""); claim != "sub" {
 		t.Fatalf("expected sub subject claim for GitHub issuer, got %s", claim)
 	}
+	if claim := issuerToChallengeClaim(IssuerTypeCIProvider, ""); claim != "sub" {
+		t.Fatalf("expected sub subject claim for CI issuer, got %s", claim)
+	}
+	if claim := issuerToChallengeClaim(IssuerTypeGitLabPipeline, ""); claim != "sub" {
+		t.Fatalf("expected sub subject claim for GitLab issuer, got %s", claim)
+	}
+	if claim := issuerToChallengeClaim(IssuerTypeCodefreshWorkflow, ""); claim != "sub" {
+		t.Fatalf("expected sub subject claim for Codefresh issuer, got %s", claim)
+	}
+	if claim := issuerToChallengeClaim(IssuerTypeChainguard, ""); claim != "sub" {
+		t.Fatalf("expected sub subject claim for Chainguard issuer, got %s", claim)
+	}
 	if claim := issuerToChallengeClaim(IssuerTypeKubernetes, ""); claim != "sub" {
 		t.Fatalf("expected sub subject claim for K8S issuer, got %s", claim)
 	}
@@ -503,45 +553,314 @@ func Test_issuerToChallengeClaim(t *testing.T) {
 }
 
 func TestToIssuers(t *testing.T) {
-	config := &FulcioConfig{
-		OIDCIssuers: map[string]OIDCIssuer{
-			"example.com": {
-				IssuerURL: "example.com",
-				ClientID:  "sigstore",
-				Type:      IssuerTypeEmail,
+	tests := []struct {
+		config *FulcioConfig
+		want   []*protobuf.OIDCIssuer
+	}{
+		{
+			config: &FulcioConfig{
+				OIDCIssuers: map[string]OIDCIssuer{
+					"example.com": {
+						IssuerURL: "example.com",
+						ClientID:  "sigstore",
+						Type:      IssuerTypeEmail,
+					},
+				},
+				MetaIssuers: map[string]OIDCIssuer{
+					"wildcard.*.example.com": {
+						ClientID: "sigstore",
+						Type:     IssuerTypeKubernetes,
+					},
+				},
+			},
+			want: []*protobuf.OIDCIssuer{
+				{
+					Audience:       "sigstore",
+					ChallengeClaim: "email",
+					Issuer: &protobuf.OIDCIssuer_IssuerUrl{
+						IssuerUrl: "example.com",
+					},
+					IssuerType: IssuerTypeEmail,
+				},
+				{
+					Audience:       "sigstore",
+					ChallengeClaim: "sub",
+					Issuer: &protobuf.OIDCIssuer_WildcardIssuerUrl{
+						WildcardIssuerUrl: "wildcard.*.example.com",
+					},
+					IssuerType: IssuerTypeKubernetes,
+				},
 			},
 		},
-		MetaIssuers: map[string]OIDCIssuer{
-			"wildcard.*.example.com": {
-				ClientID: "sigstore",
-				Type:     IssuerTypeKubernetes,
+		{
+			config: &FulcioConfig{
+				OIDCIssuers: map[string]OIDCIssuer{
+					"username.example.com": {
+						IssuerURL:     "username.example.com",
+						ClientID:      "sigstore",
+						Type:          IssuerTypeUsername,
+						SubjectDomain: "username.example.com",
+					},
+				},
+			},
+			want: []*protobuf.OIDCIssuer{
+				{
+					Audience:       "sigstore",
+					ChallengeClaim: "sub",
+					Issuer: &protobuf.OIDCIssuer_IssuerUrl{
+						IssuerUrl: "username.example.com",
+					},
+					IssuerType:    IssuerTypeUsername,
+					SubjectDomain: "username.example.com",
+				},
+			},
+		},
+		{
+			config: &FulcioConfig{
+				OIDCIssuers: map[string]OIDCIssuer{
+					"uriissuer.example.com": {
+						IssuerURL:     "uriissuer.example.com",
+						ClientID:      "sigstore",
+						Type:          IssuerTypeURI,
+						SubjectDomain: "uriissuer.example.com",
+					},
+				},
+			},
+			want: []*protobuf.OIDCIssuer{
+				{
+					Audience:       "sigstore",
+					ChallengeClaim: "sub",
+					Issuer: &protobuf.OIDCIssuer_IssuerUrl{
+						IssuerUrl: "uriissuer.example.com",
+					},
+					IssuerType:    IssuerTypeURI,
+					SubjectDomain: "uriissuer.example.com",
+				},
 			},
 		},
 	}
 
-	issuers := config.ToIssuers()
-	if len(issuers) != 2 {
-		t.Fatalf("unexpected number of issues, expected 2, got %v", len(issuers))
-	}
-
-	iss := &protobuf.OIDCIssuer{
-		Audience:       "sigstore",
-		ChallengeClaim: "email",
-		Issuer: &protobuf.OIDCIssuer_IssuerUrl{
-			IssuerUrl: "example.com",
-		},
-	}
-	if !reflect.DeepEqual(issuers[0], iss) {
-		t.Fatalf("expected issuer %v, got %v", iss, issuers[0])
-	}
-	iss = &protobuf.OIDCIssuer{
-		Audience:       "sigstore",
-		ChallengeClaim: "sub",
-		Issuer: &protobuf.OIDCIssuer_WildcardIssuerUrl{
-			WildcardIssuerUrl: "wildcard.*.example.com",
-		},
-	}
-	if !reflect.DeepEqual(issuers[1], iss) {
-		t.Fatalf("expected issuer %v, got %v", iss, issuers[1])
+	for _, test := range tests {
+		issuers := test.config.ToIssuers()
+		if !reflect.DeepEqual(issuers, test.want) {
+			t.Fatalf("expected issuers %v, got %v", test.want, issuers)
+		}
 	}
 }
+
+func TestVerifierCache(t *testing.T) {
+	cache, err := lru.New2Q[string, []*verifierWithConfig](100 /* size */)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	fc := &FulcioConfig{
+		OIDCIssuers: map[string]OIDCIssuer{
+			"issuer.dev": {
+				IssuerURL: "issuer.dev",
+				ClientID:  "sigstore",
+			},
+		},
+		verifiers: map[string][]*verifierWithConfig{},
+		lru:       cache,
+	}
+
+	// create a cache hit
+	cfg := &oidc.Config{ClientID: "sigstore"}
+	verifier := oidc.NewVerifier("issuer.dev", &mockKeySet{}, cfg)
+	fc.verifiers = map[string][]*verifierWithConfig{
+		"issuer.dev": {
+			{
+				Config:          cfg,
+				IDTokenVerifier: verifier,
+			},
+		},
+	}
+
+	// make sure we get a hit
+	v, ok := fc.GetVerifier("issuer.dev")
+	if !ok {
+		t.Fatal("unable to verifier")
+	}
+	if !reflect.DeepEqual(v, verifier) {
+		t.Fatal("got unexpected verifier")
+	}
+
+	// get verifier with SkipExpiryCheck set, should fail on cache miss
+	_, ok = fc.GetVerifier("issuer.dev", WithSkipExpiryCheck())
+	if ok {
+		t.Fatal("expected cache miss")
+	}
+
+	// create a cache hit with SkipExpiryCheck set
+	withExpiryCfg := &oidc.Config{ClientID: "sigstore", SkipExpiryCheck: true}
+	expiryVerifier := oidc.NewVerifier("issuer.dev", &mockKeySet{}, cfg)
+	fc.verifiers = map[string][]*verifierWithConfig{
+		"issuer.dev": {
+			{
+				Config:          cfg,
+				IDTokenVerifier: verifier,
+			}, {
+				Config:          withExpiryCfg,
+				IDTokenVerifier: expiryVerifier,
+			},
+		},
+	}
+	// make sure we get a hit and the correct verifier is returned
+	v, ok = fc.GetVerifier("issuer.dev", WithSkipExpiryCheck())
+	if !ok {
+		t.Fatal("unable to verifier")
+	}
+	if !reflect.DeepEqual(v, expiryVerifier) {
+		t.Fatal("got unexpected verifier")
+	}
+}
+
+func TestVerifierCacheWithCustomCA(t *testing.T) {
+	ca := &x509.Certificate{
+		SerialNumber: big.NewInt(2024),
+		Subject: pkix.Name{
+			CommonName: "Test CA",
+		},
+		NotBefore:             time.Now(),
+		NotAfter:              time.Now().AddDate(1, 0, 0),
+		IsCA:                  true,
+		KeyUsage:              x509.KeyUsageCertSign | x509.KeyUsageCRLSign,
+		BasicConstraintsValid: true,
+	}
+
+	caPrivKey, err := rsa.GenerateKey(rand.Reader, 2048)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	caBytes, err := x509.CreateCertificate(rand.Reader, ca, ca, &caPrivKey.PublicKey, caPrivKey)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	caPEM := new(bytes.Buffer)
+	pem.Encode(caPEM, &pem.Block{
+		Type:  "CERTIFICATE",
+		Bytes: caBytes,
+	})
+
+	serverCert := &x509.Certificate{
+		SerialNumber: big.NewInt(1),
+		Subject: pkix.Name{
+			CommonName: "localhost",
+		},
+		DNSNames:    []string{"localhost"},
+		IPAddresses: []net.IP{net.ParseIP("127.0.0.1")},
+		NotBefore:   time.Now(),
+		NotAfter:    time.Now().AddDate(1, 0, 0),
+		ExtKeyUsage: []x509.ExtKeyUsage{x509.ExtKeyUsageServerAuth},
+	}
+
+	serverPrivKey, err := rsa.GenerateKey(rand.Reader, 2048)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	serverCertBytes, err := x509.CreateCertificate(
+		rand.Reader,
+		serverCert,
+		ca,
+		&serverPrivKey.PublicKey,
+		caPrivKey,
+	)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	serverCertPEM := new(bytes.Buffer)
+	pem.Encode(serverCertPEM, &pem.Block{
+		Type:  "CERTIFICATE",
+		Bytes: serverCertBytes,
+	})
+
+	serverKeyPEM := new(bytes.Buffer)
+	pem.Encode(serverKeyPEM, &pem.Block{
+		Type:  "RSA PRIVATE KEY",
+		Bytes: x509.MarshalPKCS1PrivateKey(serverPrivKey),
+	})
+
+	serverTLSCert, err := tls.X509KeyPair(serverCertPEM.Bytes(), serverKeyPEM.Bytes())
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	var server *httptest.Server
+	server = httptest.NewUnstartedServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		switch r.URL.Path {
+		case "/.well-known/openid-configuration":
+			w.Header().Set("Content-Type", "application/json")
+			json.NewEncoder(w).Encode(map[string]string{
+				"issuer":   server.URL,
+				"jwks_uri": server.URL + "/keys",
+			})
+		case "/keys":
+			w.Header().Set("Content-Type", "application/json")
+			json.NewEncoder(w).Encode(map[string]interface{}{
+				"keys": []interface{}{},
+			})
+		default:
+			http.NotFound(w, r)
+		}
+	}))
+
+	server.TLS = &tls.Config{
+		Certificates: []tls.Certificate{serverTLSCert},
+	}
+	server.StartTLS()
+	defer server.Close()
+
+	cache, err := lru.New2Q[string, []*verifierWithConfig](100)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// use custom CA for OIDC issuer
+	fc := &FulcioConfig{
+		OIDCIssuers: map[string]OIDCIssuer{
+			server.URL: {
+				IssuerURL: server.URL,
+				ClientID:  "sigstore",
+				CACert:    caPEM.String(),
+			},
+		},
+		verifiers: make(map[string][]*verifierWithConfig),
+		lru:       cache,
+	}
+
+	verifier, ok := fc.GetVerifier(server.URL)
+	if !ok {
+		t.Fatal("expected to get verifier")
+	}
+	if verifier == nil {
+		t.Fatal("expected non-nil verifier")
+	}
+
+	cachedVerifier, ok := fc.GetVerifier(server.URL)
+	if !ok {
+		t.Fatal("expected to get cached verifier")
+	}
+	if !reflect.DeepEqual(verifier, cachedVerifier) {
+		t.Fatal("cached verifier doesn't match original verifier")
+	}
+
+	verifierWithOptions, ok := fc.GetVerifier(server.URL, WithSkipExpiryCheck())
+	if !ok {
+		t.Fatal("expected to get verifier with options")
+	}
+	if reflect.DeepEqual(verifier, verifierWithOptions) {
+		t.Fatal("verifier with options shouldn't match original verifier")
+	}
+}
+
+type mockKeySet struct {
+}
+
+func (m *mockKeySet) VerifySignature(_ context.Context, _ string) (payload []byte, err error) {
+	return nil, nil
+}
