@@ -17,20 +17,23 @@ package buildkite
 import (
 	"context"
 
+	"github.com/sigstore/fulcio/pkg/config"
 	"github.com/sigstore/fulcio/pkg/identity"
 	"github.com/sigstore/fulcio/pkg/identity/base"
 )
 
+// Deprecated: Use ciprovider.ciProviderIssuer instead
 type buildkiteIssuer struct {
 	identity.Issuer
 }
 
+// Deprecated: Use ciprovider.Issuer instead
 func Issuer(issuerURL string) identity.Issuer {
 	return &buildkiteIssuer{base.Issuer(issuerURL)}
 }
 
-func (e *buildkiteIssuer) Authenticate(ctx context.Context, token string) (identity.Principal, error) {
-	idtoken, err := identity.Authorize(ctx, token)
+func (e *buildkiteIssuer) Authenticate(ctx context.Context, token string, opts ...config.InsecureOIDCConfigOption) (identity.Principal, error) {
+	idtoken, err := identity.Authorize(ctx, token, opts...)
 	if err != nil {
 		return nil, err
 	}
